package mcsimul;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.applet.Applet;


/**
 * This file is part of MCSimul.
 * MCSimul is a Java application/applet, which does 2-D atomic Monte-Carlo simulation,
 * together with a graphical user interface
 * Copyright 2008, Michael Leitner
 *
 * As stated below, you are free to do whatever you want with this program, as long as
 * it conforms to the licence, but I would be glad to hear if someone found it useful,
 * so please tell me if so under michael.leitner@univie.ac.at
 *
 * MCSimul is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MCSimul is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

public class MCSimulApplet extends Applet implements Runnable, ActionListener {
    int dimx = 50, dimy = 50,attemptFrequency=10;
    private byte running;
    double anteil = .1;
    Configuration conf = new Configuration(dimx, dimy, anteil);
    MCCanvas mccanvas=new MCCanvas(conf);
    ControlPanel controlPanel=new ControlPanel(this,conf);
    java.text.DecimalFormat df;
    Thread th;
    
    @Override
    public void init() {
        java.text.DecimalFormatSymbols dummy =new java.text.DecimalFormatSymbols();
        dummy.setDecimalSeparator('.');
        df = new java.text.DecimalFormat("0.0##",dummy);  
        setBackground(Color.lightGray);
        setLayout(new FlowLayout());
        add(mccanvas);                        
        add(controlPanel);
        controlPanel.displayAF.setText(""+attemptFrequency);
        setVisible(true);
        validate();
        th = new Thread(this);
        th.start();
    }

    public void actionPerformed(ActionEvent event) {
        Object obj = event.getSource();
        if (obj instanceof Button) {
            String cmd = event.getActionCommand();
            if (cmd.equals("paint now!")) {
                try {
                    conf.paintAtoms(controlPanel.getPaintAtoms());
                    mccanvas.repaint();
                } catch (NumberFormatException e) {}
                controlPanel.rate.setText(" " + String.valueOf(df.format(conf.getRate())));
            } else {                    
                try {
                    conf.setTemp(controlPanel.getTemp());
                    conf.setU(controlPanel.getU());
                    conf.setJ(controlPanel.getJ());
                } catch (NumberFormatException e) {}           
            } 
        } else 
            if (obj.equals(controlPanel.displayAF)) {
                try {
                    attemptFrequency=controlPanel.getAttemptFrequency();
                } catch (NumberFormatException e) {}
            } 

    }

    @Override
    public void start() {
        running=1;
    }
    
    @Override 
    public void stop() {
        running=0;
    }

    public void run() {
        int i;
        while (true) {
            if ((attemptFrequency < 1)||(running==0)) {
                try {
                    Thread.sleep(100);
                } catch (InterruptedException e) {
                }
            }
            else if (attemptFrequency < 20) {
                conf.step();
                mccanvas.repaintMod();
                try {
                    Thread.sleep((int) (1000. / attemptFrequency));
                } catch (InterruptedException e) {
                }
            }
            else {
                for (i=0;i<attemptFrequency/10.;i++) conf.step();
                mccanvas.repaint();
                try {
                    Thread.sleep(100);
                } catch (InterruptedException e) {
                }
            }
        }
    }
}
